@@ -1,5 +1,6 @@
 ## check.numeric
-# RtoSQL    Convert logical R statement syntax to SQL syntax
+## RtoSQL    Convert logical R statement syntax to SQL syntax
+## check.logic.vars   Check if variables are in statement
 ## check.logic
 ## check.matchclass
 ## check.matchval
@@ -58,17 +59,17 @@ RtoSQL <- function(filter, x=NULL) {
       if (not) {
         part <- gsub("!", "", part)
 		if (grepl("%in%c", gsub(" ", "", part), ignore.case=TRUE)) {
-          part <- gsub("%in%c", " not in", gsub(" ", "", part))
+          part <- gsub("%in%c", " NOT IN", gsub(" ", "", part))
 		} else if (grepl("%in%", gsub(" ", "", part), ignore.case=TRUE)) {
 		  stop("invalid statement")
-          part <- gsub("%in%", " not in", gsub(" ", "", part))
+          part <- gsub("%in%", " NOT IN", gsub(" ", "", part))
 		}
       } else {
 		if (grepl("%in%c", gsub(" ", "", part), ignore.case=TRUE)) {
-          part <- gsub("%in%c", " in", gsub(" ", "", part))
+          part <- gsub("%in%c", " IN", gsub(" ", "", part))
 		} else if (grepl("%in%", gsub(" ", "", part), ignore.case=TRUE)) {
 		  stop("invalid statement")
-          part <- gsub("%in%", " in", gsub(" ", "", part))
+          part <- gsub("%in%", " IN", gsub(" ", "", part))
 		}
 		if (grepl(":", part)) {
 		  p1 <- strsplit(part, ":")[[1]][1]
@@ -88,10 +89,10 @@ RtoSQL <- function(filter, x=NULL) {
       post <- ifelse (is.na(strsplit(basetmp, "\\)")[[1]][2]), "", strsplit(basetmp, "\\)")[[1]][2])
 
       if (not) {
-        part <- paste0(pre, base, " is not NULL", post)
+        part <- paste0(pre, base, " IS NOT NULL", post)
         part <- gsub("!", "", part)
       } else {
-        part <- paste0(pre, base, " is NULL", post)
+        part <- paste0(pre, base, " IS NULL", post)
       }
     }
 
@@ -99,9 +100,9 @@ RtoSQL <- function(filter, x=NULL) {
   }
  
   if (grepl("&", filter)) {
-    sql <- paste(sapply(unlist(strsplit(filter, "&")), checkpart), collapse = " and ")
+    sql <- paste(sapply(unlist(strsplit(filter, "&")), checkpart), collapse = " AND ")
   } else if (grepl("\\|", filter)) {
-    sql <- paste(sapply(unlist(strsplit(sql, "\\|")), checkpart), collapse = " or ")
+    sql <- paste(sapply(unlist(strsplit(sql, "\\|")), checkpart), collapse = " OR ")
   } else {
     sql <- checkpart(filter)
   }
@@ -123,6 +124,37 @@ RtoSQL <- function(filter, x=NULL) {
 #  return(x)
 #}
 
+#' @rdname checks_desc
+#' @export
+check.logic.vars <- function(varlst, statement, ignore.case = FALSE, returnVars = FALSE) {
+  
+  ## Description: checks if variables in varlst are in statement.
+  if (all(is.null(varlst))) {
+    return(FALSE)
+  }
+  
+  chk <- sapply(varlst, function(x, statement, ignore.case) {
+    grepl(paste0("(?<!\\w)", x, "(?!\\w)"), statement, perl = TRUE, ignore.case = ignore.case)},
+    statement, ignore.case)
+  
+
+  if (any(chk)) {
+    if (returnVars) {
+      return(names(chk)[chk])
+    } else {
+      return(as.vector(chk))
+    }
+  } else {
+    if (returnVars) {
+      return(NULL)
+    } else {
+      return(FALSE)
+    }
+  }
+}
+
+
+
 #' @rdname checks_desc
 #' @export
 check.logic <- function(x, statement, filternm=NULL, stopifnull=FALSE,
@@ -152,43 +184,43 @@ check.logic <- function(x, statement, filternm=NULL, stopifnull=FALSE,
     part <- as.vector(part)
 
     ## Check if there are any variables in x that match filter
-    chk <- sum(sapply(x, function(x, y){ grepl(x, y) }, part))
+    chk <- sum(sapply(x, function(x, y){ grepl(x, y, ignore.case = TRUE) }, part))
     if (chk == 0) {
-	  return(NULL)
+	    return(NULL)
     } else {
-      chkvar <- x[sapply(x, function(x, y){ grepl(x, y) }, part)]
-	  if (grepl("\n", part)) {
-	    part <- gsub("\n", "", part)
-	  }
-	  if (grepl("\t", part)) {
-	    part <- gsub("\t", "", part)
-	  }
-	  if (grepl("!", part) && !grepl("!=", part)) {
-	    part <- gsub("!", "", part)
-	  }
-	  chklogicchar <- logic.chars[sapply(logic.chars, 
+      chkvar <- x[sapply(x, function(x, y){ grepl(x, y, ignore.case = TRUE) }, part)]
+	    if (grepl("\n", part)) {
+	      part <- gsub("\n", "", part)
+	    }
+	    if (grepl("\t", part)) {
+	      part <- gsub("\t", "", part)
+	    }
+	    if (grepl("!", part) && !grepl("!=", part)) {
+	      part <- gsub("!", "", part)
+	    }
+	    chklogicchar <- logic.chars[sapply(logic.chars, 
 	               function(x, part){ grepl(x, part, ignore.case=TRUE) }, part)]
  
-	  if (length(chklogicchar) > 1) { 
-	    if (length(chklogicchar) == 2 && all(c("notin", "in") %in% chklogicchar)) {
-	      chklogicchar <- "notin"
-	    } else if (length(chklogicchar) == 2 && all(c("NOTIN", "IN") %in% chklogicchar)) {
-	      chklogicchar <- "NOTIN"
-	    } else if (length(chklogicchar) == 3 && all(c("<", ">", "<>") %in% chklogicchar)) {
-	      chklogicchar <- "<>"
-	    } else if (length(chklogicchar) == 3 && all(c("=", "<=", "<") %in% chklogicchar)) {
-	      chklogicchar <- "<="
-	    } else if (length(chklogicchar) == 3 && all(c("=", ">=", ">") %in% chklogicchar)) {
-	      chklogicchar <- ">="
-	    } else if (length(chklogicchar) == 2 && all(c("<=", "<") %in% chklogicchar)) {
-	      chklogicchar <- "<="
-	    } else if (length(chklogicchar) == 2 && all(c(">=", ">") %in% chklogicchar)) {
-	      chklogicchar <- ">="
-	    } else if (length(chklogicchar) == 2 && all(c("<", ">") %in% chklogicchar)) {
-	      chklogicchar <- "<>"
-	    } else {
-		  stop("more than one logic char: ", toString(chklogicchar))
-		}
+	    if (length(chklogicchar) > 1) { 
+	      if (length(chklogicchar) == 2 && all(c("notin", "in") %in% chklogicchar)) {
+	        chklogicchar <- "notin"
+	      } else if (length(chklogicchar) == 2 && all(c("NOTIN", "IN") %in% chklogicchar)) {
+	        chklogicchar <- "NOTIN"
+	      } else if (length(chklogicchar) == 3 && all(c("<", ">", "<>") %in% chklogicchar)) {
+	        chklogicchar <- "<>"
+	      } else if (length(chklogicchar) == 3 && all(c("=", "<=", "<") %in% chklogicchar)) {
+	        chklogicchar <- "<="
+	      } else if (length(chklogicchar) == 3 && all(c("=", ">=", ">") %in% chklogicchar)) {
+	        chklogicchar <- ">="
+	      } else if (length(chklogicchar) == 2 && all(c("<=", "<") %in% chklogicchar)) {
+	        chklogicchar <- "<="
+	      } else if (length(chklogicchar) == 2 && all(c(">=", ">") %in% chklogicchar)) {
+	        chklogicchar <- ">="
+	      } else if (length(chklogicchar) == 2 && all(c("<", ">") %in% chklogicchar)) {
+	        chklogicchar <- "<>"
+	      } else {
+		    stop("more than one logic char: ", toString(chklogicchar))
+		  }
 	
 	    if (chklogicchar %in% c("is.na", "is.null", "IS.NA", "IS.NULL")) {
 	      partsplit <- unlist(strsplit(gsub(" ", "", part), paste0(chklogicchar, "\\(")))[[2]]
@@ -199,11 +231,11 @@ check.logic <- function(x, statement, filternm=NULL, stopifnull=FALSE,
         partmatch <- sum(partsplit %in% x)
 	    if (partmatch != 1) {
 	      return(NULL)
-		} else {
-		  if (returnvar) {
-		    return(partsplit[partsplit %in% x])
-		  }
-        }
+		  } else {
+		    if (returnvar) {
+		      return(partsplit[partsplit %in% x])
+		    }
+      }
 	  }
 	  return(part)
     }
@@ -212,22 +244,22 @@ check.logic <- function(x, statement, filternm=NULL, stopifnull=FALSE,
   ## Define function to remove odd parentheses
   remove.paren <- function(x) {
     x <- gsub(" ", "", x)
-	leftp <- sum(as.vector(gregexpr("\\(", x)[[1]]) > 0) 
-	rightp <- sum(as.vector(gregexpr("\\)", x)[[1]])> 0)
-	if (leftp > rightp) {
-	  x <- sub("\\(", "", x)
-	} else if (rightp > leftp) {
-	  x <- sub("\\)", "", x)
+	  leftp <- sum(as.vector(gregexpr("\\(", x)[[1]]) > 0) 
+	  rightp <- sum(as.vector(gregexpr("\\)", x)[[1]])> 0)
+	  if (leftp > rightp) {
+	    x <- sub("\\(", "", x)
+	  } else if (rightp > leftp) {
+	    x <- sub("\\)", "", x)
     }
-	return(x)
+	  return(x)
   }
 
   ## Return NULL if statement is NULL
   if (is.null(statement) || statement == "") {
     if (stopifnull) {
-	  stop()
-	} else {
-	  return(NULL)
+	    stop()
+	  } else {
+	    return(NULL)
     }
   }
 
@@ -271,13 +303,13 @@ check.logic <- function(x, statement, filternm=NULL, stopifnull=FALSE,
     grept <- Rlogic.chars.diff[unlist(sapply(Rlogic.chars.diff,
 		          function(x, statement){grepl(x, statement, ignore.case=TRUE)}, statement))]
 				  
-	if (length(grept) > 0) {
-	  if (syntax == "SQL") {
-	    message("syntax is R")
-		statement <- RtoSQL(statement)
-	  }
-	  syntax <- "R"
-	} 
+	  if (length(grept) > 0) {
+	    if (syntax == "SQL") {
+	      #message("syntax is R")
+		    statement <- RtoSQL(statement)
+	    }
+	    syntax <- "R"
+	  } 
     if (grepl("&", statement, ignore.case=TRUE) || grepl("\\|", statement, ignore.case=TRUE)) {
       syntax <- "R"
     } else if (grepl(" and ", statement, ignore.case=TRUE) || grepl(" or ", statement, ignore.case=TRUE)) {
@@ -290,7 +322,7 @@ check.logic <- function(x, statement, filternm=NULL, stopifnull=FALSE,
     } else if (syntax == "SQL") {
       logic.chars <- SQLlogic.chars
     }
-	if (syntax == "R") {
+	  if (syntax == "R") {
 #      if (grepl("==", statement) && sum(gregexpr(equalsign, statement)>0) == 0) {
 #        message("must be R syntax.. changing = to ==")
 #        statement <- gsub("=", "==", statement)
@@ -301,37 +333,37 @@ check.logic <- function(x, statement, filternm=NULL, stopifnull=FALSE,
       if (grepl("\\|\\|", statement)) {
         statement <- gsub("\\|\\|", "\\|", statement)
       }
-	}
+	  }
 
     ## Check parentheses
     paren.left <- sum(attr(gregexpr("\\(", statement)[[1]], "match.length") > 0)
     paren.right <- sum(attr(gregexpr("\\)", statement)[[1]], "match.length") > 0)
     if (paren.left < paren.right) {
-	  message("invalid logical statement... missing left parenthesis")
-	  if (stopifinvalid) {
+	    message("invalid logical statement... missing left parenthesis")
+	    if (stopifinvalid) {
         stop()
-	  } else {
-	    return(NULL)
-	  }
+	    } else {
+	      return(NULL)
+	    }
     } else if (paren.left > paren.right) {
-	  message("invalid logical statement... missing right parenthesis")
-	  if (stopifinvalid) {
+	    message("invalid logical statement... missing right parenthesis")
+	    if (stopifinvalid) {
         stop("invalid logical statement... missing right parenthesis")
-	  } else {
-	    return(NULL)
-	  }
+	    } else {
+	      return(NULL)
+	    }
     } 
 
-	#statement <- gsub(" ", "", statement)
-	if (syntax == "R") {
-	  andnm <- "&"
+	  #statement <- gsub(" ", "", statement)
+	  if (syntax == "R") {
+	    andnm <- "&"
       ornm <- "\\|"
-	} else {
+	  } else {
       andnm <- "^and$"
-	  ornm <- "^or$"
+	    ornm <- "^or$"
       andnm <- ifelse(grepl(" and ", statement), " and ", " AND ")
       ornm <- ifelse(grepl(" or ", statement), " or ", " OR ")	  
-	}
+	  }
 
     if (grepl(andnm, statement) && grepl(ornm, statement)) {
       partsAND <- trimws(unlist(strsplit(statement, andnm)[[1]]))
@@ -341,10 +373,10 @@ check.logic <- function(x, statement, filternm=NULL, stopifnull=FALSE,
 		    sum(attr(gregexpr("\\)", partsAND)[[1]], "match.length"))) {
 
         ## Split AND parts
-		parts <- unlist(sapply(partsAND, function(x) strsplit(x, ornm)))
+		    parts <- unlist(sapply(partsAND, function(x) strsplit(x, ornm)))
 		  
-		## Remove odd parentheses
-		parts <- sapply(as.vector(parts), remove.paren)
+		    ## Remove odd parentheses
+		    parts <- sapply(as.vector(parts), remove.paren)
 
         ## Check if there are any variables in x that match filter
         chkparts <- sapply(parts, chkpartnm, x, logic.chars)
@@ -353,10 +385,10 @@ check.logic <- function(x, statement, filternm=NULL, stopifnull=FALSE,
 		    sum(attr(gregexpr("\\)", partsOR)[[1]], "match.length"))) {
 			
         ## Split OR parts 
-		parts <- unlist(sapply(partsOR, function(x) strsplit(x, andnm)))
+		    parts <- unlist(sapply(partsOR, function(x) strsplit(x, andnm)))
 		  
-		## Remove odd parentheses
-		parts <- sapply(as.vector(parts), remove.paren)
+		    ## Remove odd parentheses
+		    parts <- sapply(as.vector(parts), remove.paren)
 
         ## Check if there are any variables in x that match filter
         chkparts <- sapply(parts, chkpartnm, x, logic.chars, returnvar)
@@ -377,25 +409,25 @@ check.logic <- function(x, statement, filternm=NULL, stopifnull=FALSE,
       ## Check if there are any variables in x that match filter
       chkparts <- sapply(parts, chkpartnm, x, logic.chars, returnvar)
 
-	} else {
+	  } else {
 	
-	  chkparts <- chkpartnm(statement, x, logic.chars, returnvar)
-	}
+	    chkparts <- chkpartnm(statement, x, logic.chars, returnvar)
+	  }
 
     if (is.null(chkparts) || any(sapply(chkparts, is.null))) {
-	  message(fwarning)
-	  if (returnpart) { 
-	    if (all(sapply(chkparts, is.null))) {
-		  if (stopifinvalid) {
+	    message(fwarning)
+	    if (returnpart) { 
+	      if (all(sapply(chkparts, is.null))) {
+		      if (stopifinvalid) {
             stop()
           } else {
             return(NULL)
           }
-		}
-	    if (sum(sapply(chkparts, is.null)) > 0 && sum(sapply(chkparts, is.null)) < length(chkparts)) {
-		  return(names(chkparts)[!sapply(chkparts, is.null)])
-		}
-	  }
+		    }
+	      if (sum(sapply(chkparts, is.null)) > 0 && sum(sapply(chkparts, is.null)) < length(chkparts)) {
+		      return(names(chkparts)[!sapply(chkparts, is.null)])
+		    }
+	    }
       if (stopifinvalid) {
         stop()
       } else {
@@ -407,7 +439,7 @@ check.logic <- function(x, statement, filternm=NULL, stopifnull=FALSE,
   if (returnvar) {
     return(unlist(names(chkparts))) 
   } else {
-    message(statement)
+    #message(statement)
     return(statement)
   }
 }
@@ -415,7 +447,8 @@ check.logic <- function(x, statement, filternm=NULL, stopifnull=FALSE,
 
 #' @rdname checks_desc
 #' @export
-check.matchclass <- function(tab1, tab2, matchcol, var2=NULL, tab1txt=NULL, tab2txt=NULL) {
+check.matchclass <- function(tab1, tab2, matchcol, var2=NULL, 
+                             tab1txt=NULL, tab2txt=NULL) {
   ## Description: check that the class of matchcol in tab2 matches class in tab1
   ## 	  If different, changes class2 to class1
   ##	  For multiple variables with same name in both tables,
@@ -489,15 +522,24 @@ check.matchclass <- function(tab1, tab2, matchcol, var2=NULL, tab1txt=NULL, tab2
       }
     } else if (inherits(tab1[[v1]], what = "factor")) {
       v1levels <- levels(tab1[[v1]])
+
       if (!all(unique(tab2[[v2]]) %in% v1levels)) {
         misslevel <- unique(tab2[[v2]])[!unique(tab2[[v2]]) %in% v1levels]
-        message("missing level values: ", toString(misslevel), "... returning NA factor values")
+        
+        #if (any(is.na(misslevel)) && factor.addNA) {
+        #  tab2[[v2]] <- addNA(tab2[[v2]])
+        #  v1levels <- levels(tab2[[v2]])
+        #}
+      
+        if (any(!is.na(misslevel))) {
+          message("missing level values: ", toString(misslevel), "... returning NA factor values")
+        }
       }
       if (all(v1levels %in% tab2[[v2]])) {
-        tab2[[v2]] <- factor(tab2[[v2]], levels=v1levels)
+        tab2[[v2]] <- factor(tab2[[v2]], levels=v1levels, exclude = NULL)
       } else {
         #message("missing factor levels")
-        tab2[[v2]] <- factor(tab2[[v2]], levels=v1levels)
+        tab2[[v2]] <- factor(tab2[[v2]], levels=v1levels, exclude = NULL)
       }
     } else if (inherits(tab2[[v2]], what = "factor")) {
       tab2[[v2]] <- as.character(tab2[[v2]])
@@ -653,3 +695,54 @@ check.matchval <- function(tab1, tab2, var1, var2=NULL, tab1txt=NULL, tab2txt=NU
 }
 
 
+
+#' @rdname checks_desc
+#' @export
+check.unique <- function(x, xvar, 
+                         uniquex, 
+                         NAname = "Other",
+                         addfactors = TRUE) {
+  
+  ## DESCRIPTION: add factor levels to uniquex if missing
+  
+  
+  ## add NA factors if NA values are in dataset
+  if (any(is.na(as.character(x[[xvar]])))) {
+    if (ncol(uniquex) == 1) {
+      if (!any(is.na(as.vector(uniquex[[xvar]])))) {
+        uniquex <- rbind(uniquex, list(NA))
+      }
+      if (!any(is.na(levels(uniquex[[xvar]])))) {
+        uniquex[[xvar]] <- addNA(uniquex[[xvar]])
+      }
+    } else {
+      if (!is.null(NAname)) {
+        
+        
+        if (any(is.na(as.character(uniquex[[xvar]])))) {
+          ## check if name exists... if exists, change name to 'NAname'_1
+          NAname <- checknm(NAname, uniquex[[xvar]])
+          
+          uniquex[is.na(as.character(uniquex[[xvar]]))][[xvar]] <- NAname
+        } else {
+          ## check if name exists... if exists, change name to 'NAname'_1
+          notxvar <- names(uniquex)[names(uniquex) != xvar]
+          if (length(notxvar) != 1) {
+            stop("only 2 columns allowed")
+          }
+          NAname <- checknm(NAname, uniquex[[notxvar]])
+          
+          uniquex <- rbind(uniquex, list(NA, NAname))
+          uniquex[[xvar]] <- addNA(uniquex[[xvar]])
+        }
+      }
+    }
+  }
+  ## check other missing factor values and add if necessary
+  #xvalues <- sort(unique(x[[xvar]]))
+  #uniquexlevels <- sort(levels(uniquex[[xvar]]))
+  
+
+  return(uniquex)
+}
+
