@@ -489,20 +489,44 @@ nbrdigits <- function(x) {
   max(nchar(sapply(strsplit(as.character(x), "\\."), "[[", 1)), na.rm=TRUE)
 }
 
+
+
 #' @rdname internal_desc
 #' @export
-getfilter <- function(att, val, syntax="R") {
-## DESCRIPTION: create filter string from att and val
-## syntax - ('R', 'sql')
-  if (is.character(val)) {
-    val <- encodeString(val, quote="'")
+getfilter <- function(att, val, syntax = "R", quote = FALSE, like = FALSE) {
+  ## DESCRIPTION: create filter string from att and val
+  ## syntax - ('R', 'sql')
+  
+  if (!syntax %in% c("R", "sql")) {
+    stop("Invalid syntax. Must be either 'R' or 'sql'")
   }
-  filter <- paste0(att, " %in% c(", toString(val), ")")
-
-  if (syntax == 'sql') {
-    filter <- gsub("%in% c", "in", filter)
+  
+  if (like) {
+    
+    if (!is.character(val)) {
+      stop("'like' functionality only works with vals(s) of type character")
+    }
+    if (syntax == "R") {
+      valstr <- encodeString(paste0(val, collapse = "|"), quote = "'")
+      filt <- paste0("grepl(", valstr, ", ", att, ")")
+    } else {
+      filt <- paste0(att, " LIKE ", "'%", val, "%'", collapse = " OR ")
+    }
+    
+  } else {
+    if (is.character(val) || quote) {
+      val <- encodeString(val, quote = "'")
+    } 
+    filt <- paste0(att, " %in% c(", toString(val), ")")
+    
+    if (syntax == 'sql') {
+      filt <- gsub("%in% c", "IN ", filt)
+    }
+    
   }
-  return(filter)
+  
+  return(filt)
+  
 }
 
 #' @rdname internal_desc
@@ -708,23 +732,6 @@ date2char <- function(df, col, formatstr = '%Y-%m-%d') {
 }
 
 
-#' @rdname internal_desc
-#' @export
-getfilter <- function(att, val, syntax="R", quote=FALSE) {
-## DESCRIPTION: create filter string from att and val
-## syntax - ('R', 'sql')
-  if (is.character(val) || quote) {
-    val <- encodeString(val, quote="'")
-  } 
-  filter <- paste0(att, " %in% c(", toString(val), ")")
-
-  if (syntax == 'sql') {
-    filter <- gsub("%in% c", "IN", filter)
-  }
-  return(filter)
-}
-
-
 #' @rdname internal_desc
 #' @export
 checklevels <- function(x, uniquex, xvar, keepNA=TRUE) {
