@@ -10,6 +10,8 @@
 #' index.
 #' @param index String. Name of variable(s) in dfobj to make (non-unique)
 #' index.
+#' @param lowernames Logical. If TRUE, convert column names to lowercase 
+#' before writing to output.
 #' dbconnopen Logical. If TRUE, keep database connection open.
 #' @param savedata_opts List. See help(savedata_options()) for a list
 #' of options.
@@ -29,9 +31,10 @@ datExportData <- function(dfobj,
                           create_dsn = FALSE,
                           index.unique = NULL, 
                           index = NULL, 
+                          lowernames = FALSE,
                           savedata_opts = savedata_options(),
                           dbconn = NULL,
-                          dbconnopen = FALSE
+                          dbconnopen = TRUE
                           ) {
   ###########################################################################
   ## DESCRIPTION: Exports a data.frame to file or database.
@@ -93,19 +96,25 @@ datExportData <- function(dfobj,
       assign(names(savedata_opts)[[i]], savedata_opts[[i]])
     }
   }
- 
+
+  if (is.null(dbconn) && !is.null(outconn)) {
+    dbconn <- outconn
+  }
+  
   ## Check output data
-  outlst <- pcheck.output(out_fmt=out_fmt, outfolder=outfolder,
-	out_dsn=out_dsn, overwrite_dsn=overwrite_dsn, overwrite_layer=overwrite_layer,
-	outfn.date=outfn.date, add_layer=add_layer, append_layer=append_layer,
-	outfn.pre=outfn.pre, out_conn=dbconn, dbconnopen=TRUE)
+  outlst <- pcheck.output(out_fmt = out_fmt, outfolder = outfolder,
+	                  out_dsn=out_dsn, overwrite_dsn = overwrite_dsn, 
+	                  overwrite_layer = overwrite_layer, outfn.date = outfn.date, 
+	                  add_layer = add_layer, append_layer = append_layer,
+	                  outfn.pre = outfn.pre, outconn = dbconn, dbconnopen = TRUE)
   out_fmt <- outlst$out_fmt
   out_dsn <- outlst$out_dsn
   outfolder <- outlst$outfolder
   overwrite_layer <- outlst$overwrite_layer
   append_layer <- outlst$append_layer
-  out_conn <- outlst$out_conn
- 
+  outconn <- outlst$outconn
+  
+
   ## Check out_layer
   ####################################################
   if (is.null(out_dsn) && is.null(out_layer)) {
@@ -124,11 +133,15 @@ datExportData <- function(dfobj,
   if (out_fmt %in% c("sqlite", "gpkg")) {
     gpkg <- ifelse(out_fmt == "gpkg", TRUE, FALSE)
  
-    dbconn <- write2sqlite(setDT(dfobj), SQLitefn=out_dsn, 
-          out_name=out_layer, gpkg=gpkg,
-          overwrite=overwrite_layer, append_layer=append_layer,
-          index.unique=index.unique, index=index, createnew=create_dsn, 
-          dbconn=out_conn, dbconnopen=TRUE)
+    dbconn <- write2sqlite(setDT(dfobj), 
+                           SQLitefn = out_dsn, 
+                           lowernames = lowernames,
+                           out_name = out_layer, gpkg = gpkg, 
+                           overwrite = overwrite_layer, 
+                           append_layer = append_layer,
+                           index.unique = index.unique, 
+                           index = index, createnew = create_dsn, 
+                           dbconn = outconn, dbconnopen = TRUE)
 
   } else if (out_fmt == "gdb") {
      stop("cannot write to a geodatabase")
@@ -139,21 +152,30 @@ datExportData <- function(dfobj,
 #			overwrite=overwrite_layer)
 
   } else if (out_fmt == "csv") {
-    write2csv(dfobj, outfolder=outfolder, outfilenm=out_layer,
-		outfn.pre=outfn.pre, outfn.date=outfn.date, overwrite=overwrite_layer,
-		appendfile=append_layer)
+    write2csv(dfobj, 
+              lowernames = lowernames,
+              outfolder = outfolder, outfilenm = out_layer, 
+              outfn.pre = outfn.pre, outfn.date = outfn.date, 
+              overwrite = overwrite_layer, appendfile = append_layer)
 
   } else if (out_fmt == "rda") {
-    objfn <- getoutfn(outfn=out_layer, outfolder=outfolder, outfn.pre=outfn.pre,
-		outfn.date=outfn.date, overwrite=overwrite_layer, ext="rda")
+    objfn <- getoutfn(outfn = out_layer, 
+                      outfolder = outfolder, 
+                      outfn.pre = outfn.pre, 
+                      outfn.date = outfn.date, 
+                      overwrite = overwrite_layer, ext =" rda")
     save(dfobj, file=objfn)
   } else if (out_fmt == "rds") {
-    objfn <- getoutfn(outfn=out_layer, outfolder=outfolder, outfn.pre=outfn.pre,
-		outfn.date=outfn.date, overwrite=overwrite_layer, ext="rds")
+    objfn <- getoutfn(outfn=out_layer, 
+                      outfolder=outfolder, 
+                      outfn.pre=outfn.pre, outfn.date = outfn.date, 
+                      overwrite = overwrite_layer, ext = "rds")
     save(dfobj, file=objfn)
   } else if (out_fmt == "llo") {
-    objfn <- getoutfn(outfn=out_layer, outfolder=outfolder, outfn.pre=outfn.pre,
-		outfn.date=outfn.date, overwrite=overwrite_layer, ext="llo")
+    objfn <- getoutfn(outfn = out_layer, 
+                      outfolder = outfolder, 
+                      outfn.pre = outfn.pre, outfn.date = outfn.date, 
+                      overwrite = overwrite_layer, ext = "llo")
     save(dfobj, file=objfn)
   } else {
     stop(out_fmt, " currently not supported")
