@@ -76,22 +76,23 @@ crosstabx <- function(x, xvar, estnm, psenm, allin1=FALSE, char.width=NULL,
       char.width <- max(nchar(na.omit(x[[psenm]])))
     }
     estpse <- mapply(allin1f, x=x[[estnm]], y=x[[psenm]],
-		MoreArgs=list(char.width=char.width, estnull=estnull, psenull=psenull,
-		estround=estround, pseround=pseround))
+		       MoreArgs=list(char.width = char.width, 
+		                     estnull = estnull, psenull = psenull,
+		                     estround = estround, pseround = pseround))
     names(estpse) <- x[[xvar]]
 	
-	if (is.factor(x[[xvar]])) {
-	  xvar.levels <- levels(x[[xvar]])
-	  if (length(estpse) < length(xvar.levels)) {
-	    misslevels <- xvar.levels[!xvar.levels %in% names(estpse)]
-		missvect <- sapply(rep(0, length(misslevels)), allin1f, 0, 
-		           char.width=char.width, estnull=estnull, psenull=psenull)	
+	  if (is.factor(x[[xvar]])) {
+	    xvar.levels <- levels(x[[xvar]])
+	    if (length(estpse) < length(xvar.levels)) {
+	      misslevels <- xvar.levels[!xvar.levels %in% names(estpse)]
+		    missvect <- sapply(rep(0, length(misslevels)), allin1f, 0, 
+		                char.width=char.width, estnull=estnull, psenull=psenull)	
         names(missvect) <- misslevels				   
 
-		estpse <- c(estpse, missvect)
-		estpse <- estpse[order(xvar.levels)]
-	  }  
-    }
+		    estpse <- c(estpse, missvect)
+		    estpse <- estpse[order(xvar.levels)]
+	    }  
+	  }
     return(estpse)
 	
   } else {
@@ -143,15 +144,18 @@ add0unit <- function(x, xvar, uniquex, unitvar=NULL, xvar.add0=FALSE,
     if (is.null(uniquex2)) stop("must include uniquex2")
     if (!"data.table" %in% class(uniquex2)) {
       uniquex2 <- setDT(uniquex2)
-	}
+	  }
     byvars <- c(byvars, xvar2)
 
     if (xvar.add0 && xvar2.add0) {
-       uniquex.exp <- unique(expand.grid(uniquex[[xvar]], uniquex2[[xvar2]],
-		stringsAsFactors=FALSE))
+      
+      ## Expand uniquex to all combinations
+      uniquex.exp <- unique(expand.grid(uniquex[[xvar]], uniquex2[[xvar2]],
+		                    stringsAsFactors=FALSE))
       if (!is.null(unitvar)) {
-        uniquex.exp <- data.table(uvar=rep(unique(x[[unitvar]]),
-			each=nrow(uniquex.exp)), uniquex.exp)
+        uniquex.exp <- 
+          data.table(uvar = rep(unique(x[[unitvar]]),
+			               each = nrow(uniquex.exp)), uniquex.exp)
         setnames(uniquex.exp, c(unitvar, xvar, xvar2))
         chkvars <- c(unitvar, xvar, xvar2)
       } else {
@@ -159,79 +163,86 @@ add0unit <- function(x, xvar, uniquex, unitvar=NULL, xvar.add0=FALSE,
         chkvars <- c(xvar, xvar2)
       }
       ordvars <- chkvars
- 
+      
+      ## Match classes of xvar in x with xvar in uniquex
+      xchk <- check.matchclass(uniquex, uniquex.exp, xvar)
+      uniquex <- xchk$tab1
+      uniquex.exp <- xchk$tab2
+      
+      ## Match classes of xvar2 in x with xvar2 in uniquex2
+      xchk <- check.matchclass(uniquex2, uniquex.exp, xvar2)
+      uniquex2 <- xchk$tab1
+      uniquex.exp <- xchk$tab2
+      
+      ## Merge uniquex to uniquex.exp if greater than 1 column
       if (ncol(uniquex) > 1) {
-        xchk <- check.matchclass(uniquex.exp, uniquex, xvar)
-        uniquex.exp <- xchk$tab1
-        uniquex <- xchk$tab2
         uniquex.exp <- merge(uniquex.exp, uniquex, by=xvar)
         ordvars <- c(names(uniquex), xvar2)
       }
 
+      ## Merge uniquex2 to uniquex.exp if greater than 1 column
       if (ncol(uniquex2) > 1) {
-        xchk <- check.matchclass(uniquex.exp, uniquex2, xvar2)
-        uniquex.exp <- xchk$tab1
-        uniquex2 <- xchk$tab2
         uniquex.exp <- merge(uniquex.exp, uniquex2, by=xvar2)
         ordvars <- c(names(uniquex), names(uniquex2))
       }
 
+      ## Match classes of chkvars in x with checkvars in uniquex.exp
       xchk <- check.matchclass(uniquex.exp, x, chkvars)
       uniquex.exp <- xchk$tab1
       x <- xchk$tab2
 
+      ## Merge uniquex.exp to x
       x <- merge(uniquex.exp, x, by=chkvars, all.x=TRUE)
       x[is.na(x)] <- 0
 
     } else if (xvar.add0) {
 
+      ## Expand uniquex to all combinations
       if (!is.null(unitvar)) {
         uniquex.exp <- unique(x[, expand.grid(uniquex[[xvar]], get(xvar2)), by=unitvar])
-        #uniquex.exp2 <- unique(x[, expand.grid(as.character(uniquex[[xvar]]), get(xvar2)), by=unitvar])
         setnames(uniquex.exp, c(unitvar, xvar, xvar2))
         chkvars <- c(unitvar, xvar, xvar2)
-
       } else {
         uniquex.exp <- unique(x[, expand.grid(uniquex[[xvar]], get(xvar2))])
         setnames(uniquex.exp, c(xvar, xvar2))
         chkvars <- c(xvar, xvar2)
       }
-
+      
+      ## Match classes of xvar in x with xvar in uniquex
+      xchk <- check.matchclass(uniquex, uniquex.exp, xvar)
+      uniquex <- xchk$tab1
+      uniquex.exp <- xchk$tab2
+      
+      ## Match classes of xvar2 in x with xvar2 in uniquex2
+      xchk <- check.matchclass(uniquex2, uniquex.exp, xvar2)
+      uniquex2 <- xchk$tab1
+      uniquex.exp <- xchk$tab2
+      
+      ## Merge uniquex to uniquex.exp if greater than 1 column
       if (ncol(uniquex) > 1) {
-        xchk <- check.matchclass(uniquex.exp, uniquex, xvar)
-        uniquex.exp <- xchk$tab1
-        uniquex <- xchk$tab2
-		
         uniquex.exp <- merge(uniquex.exp, uniquex, by=xvar, all.x=TRUE, all.y=TRUE)
       }
+      
+      ## Merge uniquex2 to uniquex.exp if greater than 1 column
       if (ncol(uniquex2) > 1) {
-	    ## Merge uniquex
-        xchk <- check.matchclass(uniquex.exp, uniquex2, xvar2)
-        uniquex.exp <- xchk$tab1
-        uniquex2 <- xchk$tab2
-
         uniquex.exp <- merge(uniquex.exp, uniquex2, by=xvar2, all.x=TRUE)
       }
 
-      ## Merge uniquex
+      ## Match classes of chkvars in x with checkvars in uniquex.exp
       xchk <- check.matchclass(uniquex.exp, x, chkvars)
       uniquex.exp <- xchk$tab1
       x <- xchk$tab2
-
-      ## Merge uniquex.exp
+      
+      ## Merge uniquex.exp to x
       x <- unique(merge(x, uniquex.exp, by=chkvars, all.y=TRUE))
       setcolorder(x, c(chkvars, names(x)[!names(x) %in% chkvars]))
 
-      #setnames(x, unitvar, "uvar")
-      #x <- x[uniquex[rep(1:nrow(uniquex.exp), uniqueN(x$uvar)),
-	 #	c(.SD, list(uvar=rep(unique(x$uvar), each=nrow(uniquex.exp))))],
-	 #	on=c("uvar", xvar)]
-
       ## Set NA values to 0
       x[is.na(x)] <- 0
 
     } else if (xvar2.add0) {
 	
+      ## Expand uniquex to all combinations
       if (!is.null(unitvar)) {
         uniquex.exp <- unique(x[, expand.grid(uniquex2[[xvar2]], get(xvar)), by=unitvar])
         setnames(uniquex.exp, c(unitvar, xvar2, xvar))
@@ -241,52 +252,60 @@ add0unit <- function(x, xvar, uniquex, unitvar=NULL, xvar.add0=FALSE,
         setnames(uniquex.exp, c(xvar2, xvar))
         chkvars <- c(xvar, xvar2)
       }
-
+      
+      ## Match classes of xvar in x with xvar in uniquex
+      xchk <- check.matchclass(uniquex, uniquex.exp, xvar)
+      uniquex <- xchk$tab1
+      uniquex.exp <- xchk$tab2
+      
+      ## Match classes of xvar2 in x with xvar2 in uniquex2
+      xchk <- check.matchclass(uniquex2, uniquex.exp, xvar2)
+      uniquex2 <- xchk$tab1
+      uniquex.exp <- xchk$tab2
+      
+      
+      ## Merge uniquex to uniquex.exp if greater than 1 column
       if (ncol(uniquex) > 1) {
-        ## Merge uniquex.exp
-        xchk <- check.matchclass(uniquex.exp, uniquex, xvar)
-        uniquex.exp <- xchk$tab1
-        uniquex <- xchk$tab2
-	  
         uniquex.exp <- merge(uniquex.exp, uniquex, by=xvar)
       }
+      
+      ## Merge uniquex2 to uniquex.exp if greater than 1 column
       if (ncol(uniquex2) > 1) {
-        xchk <- check.matchclass(uniquex.exp, uniquex2, xvar2)
-        uniquex.exp <- xchk$tab1
-        uniquex2 <- xchk$tab2
-
         uniquex.exp <- merge(uniquex.exp, uniquex2, by=xvar2, all.y=TRUE)
       }
 
-      ## Merge uniquex.exp
+      ## Match classes of x with uniquex.exp
       xchk <- check.matchclass(uniquex.exp, x, chkvars)
       uniquex.exp <- xchk$tab1
       x <- xchk$tab2
-
-      ## Merge uniquex.exp
+      
+      ## Merge uniquex.exp to x
       x <- merge(uniquex.exp, x, by=chkvars, all.x=TRUE, all.y=TRUE)
 
-      #setnames(x, unitvar, "uvar")
-      #x <- x[uniquex[rep(1:nrow(uniquex.exp), uniqueN(x$uvar)),
-	 #	c(.SD, list(uvar=rep(unique(x$uvar), each=nrow(uniquex.exp))))],
-	 #	on=c("uvar", xvar)]
-
       ## Set NA values to 0
       x[is.na(x)] <- 0
 
     } else {
 	
+      ## Match classes of xvar in x with xvar in uniquex
       xchk <- check.matchclass(uniquex, x, xvar)
       uniquex <- xchk$tab1
       x <- xchk$tab2
+      
+      ## Merge uniquex to x
       x <- merge(uniquex, x, by=xvar, all.y=TRUE)
-
+      
+      ## Match classes of xvar2 in x with xvar2 in uniquex2
       xchk <- check.matchclass(uniquex2, x, xvar2)
       uniquex2 <- xchk$tab1
       x <- xchk$tab2
+      
+      ## Merge uniquex2 to x
       x <- merge(uniquex2, x, by=xvar2, all.y=TRUE)
     }
+    
   } else {  ## is.null(xvar2)
+    
     if (xvar.add0) {
       xchk <- check.matchclass(uniquex, x, byvars)
       uniquex <- xchk$tab1
@@ -295,26 +314,51 @@ add0unit <- function(x, xvar, uniquex, unitvar=NULL, xvar.add0=FALSE,
       if (!is.null(unitvar)) {
         setnames(x, unitvar, "uvar")
         x <- x[uniquex[rep(1:nrow(uniquex), uniqueN(x$uvar)),
-		c(.SD, list(uvar=rep(unique(x$uvar), each=nrow(uniquex))))],
-		on=c("uvar", xvar)]
+		            c(.SD, list(uvar=rep(unique(x$uvar), each=nrow(uniquex))))],
+		            on=c("uvar", xvar)]
         setnames(x, "uvar", unitvar)
         x[is.na(x)] <- 0
       } else {
         x <- merge(uniquex, x, by=byvars, all.y=TRUE)
       }
     } else {
-
+     
+      ## Match classes of byvars in x with byvars in uniquex
       xchk <- check.matchclass(uniquex, x, byvars)
       uniquex <- xchk$tab1
       x <- xchk$tab2
-
+      
+      ## Merge uniquex to x
       x <- merge(uniquex, x, by=byvars, all.y=TRUE)
     }
   }
 
-  if (is.factor(uniquex[[xvar]])) {
-    x[[xvar]] <- factor(x[[xvar]], levels=levels(uniquex[[xvar]]))
-  }
+  # ## Set factor levels 
+  # if (is.factor(uniquex[[xvar]])) {
+  #   xlevels <- levels(uniquex[[xvar]])
+  #   if (is.factor(x[[xvar]])) {
+  #     setattr(x[[xvar]], "levels", xlevels)
+  #   } else {
+  #     x[[xvar]] <- factor(x[[xvar]])
+  #     setattr(x[[xvar]], "levels", xlevels)
+  #   }
+  # } else {
+  #   x[[xvar]] <- factor(x[[xvar]], levels=uniquex[[xvar]])
+  # }
+  # 
+  # if (!is.null(xvar2)) {
+  #   if (is.factor(uniquex2[[xvar2]])) {
+  #     xlevels2 <- levels(uniquex2[[xvar2]])
+  #     if (is.factor(x[[xvar2]])) {
+  #       setattr(x[[xvar2]], "levels", xlevels2)
+  #     } else {
+  #       x[[xvar2]] <- factor(x[[xvar2]])
+  #       setattr(x[[xvar2]], "levels", xlevels2)
+  #     }
+  #   } else {
+  #     x[[xvar2]] <- factor(x[[xvar2]], levels=uniquex2[[xvar2]])
+  #   }
+  # }
 
   if (is.null(unitvar)) {
     ordervars <- byvars
@@ -444,22 +488,21 @@ crossxtab <- function (group.est, rowvar.est=NULL, colvar.est=NULL, total.est=NU
 
   ## Generate tables
   #################################################################################
-  #est <- xtabs(get(estnm) ~ get(rowvar) + get(colvar), group.est)
-  #pse <- xtabs(get(psenm) ~ get(rowvar) + get(colvar), group.est)
-
   if (rowgrp) {
     est <- dcast(group.est, get(rowgrpnm) + get(rowvar) ~ get(colvar),
-		fun.aggregate=I, value.var=estnm, fill=estnull)
+                 value.var = estnm, fill = estnull, drop = FALSE)
     pse <- dcast(group.est, get(rowgrpnm) + get(rowvar) ~ get(colvar),
-		fun.aggregate=I, value.var=psenm, fill=psenull)
+                 value.var = psenm, fill = psenull, drop = FALSE)
     crnames <- c("rowgrpnm", "rowvar")
   } else {
     est <- dcast(group.est, get(rowvar) ~ get(colvar),
-		fun.aggregate=I, value.var=estnm, fill=estnull, drop=FALSE)
+                 value.var = estnm, fill = estnull, drop = FALSE)
     pse <- dcast(group.est, get(rowvar) ~ get(colvar),
-		fun.aggregate=I, value.var=psenm, fill=psenull, drop=FALSE)
+                 value.var = psenm, fill = psenull, drop = FALSE)
+    
     crnames <- "rowvar"
   }
+
   est <- est[, lapply(.SD, unAsIs)]
   pse <- pse[, lapply(.SD, unAsIs)]
 
@@ -483,30 +526,30 @@ crossxtab <- function (group.est, rowvar.est=NULL, colvar.est=NULL, total.est=NU
   if (allin1) {
     #estmat <- as.matrix(est[, -(1:rnbr)])
     #psemat <- as.matrix(pse[, -(1:rnbr)])
-
-    estall1 <- mapply(allin1f, estmat, psemat, MoreArgs=list(char.width=char.width,
-		estnull=estnull, psenull=psenull, estround=estround, pseround=pseround))
+    estall1 <- mapply(allin1f, estmat, psemat, 
+                      MoreArgs = list(char.width = char.width,
+		                                  estnull = estnull, psenull = psenull, 
+		                                  estround = estround, pseround = pseround))
     estpse <- data.table(est[, 1:rnbr], matrix(estall1, nrow(estmat), ncol(estmat)))
     names(estpse) <- cnames
   } 
- 
+
   if (is.null(colvar.est) || is.null(rowvar.est)) {
     if (!is.null(colvar.est)) {
       estpse.col <- crosstabx(colvar.est, colvar, estnm, psenm, allin1=allin1,
-		char.width=char.width, estnull=estnull, psenull=psenull,
-		estround=estround, pseround=pseround)
-	  estpse.col <- estpse.col[group.est[[colvar]]]
-
+		            char.width=char.width, estnull=estnull, psenull=psenull,
+		            estround=estround, pseround=pseround)
       if (allin1) {
         estpse <- rbind(setDF(estpse), c(totals, estpse.col))
       } else {
+        estpse.col$est <- estpse.col$est[unique(group.est[[colvar]])]
         est <- rbind(setDF(est), c(totals, estpse.col$est))
         pse <- rbind(setDF(pse), c(totals, estpse.col$pse))
       }
     } else if (!is.null(rowvar.est)) {
       estpse.row <- crosstabx(rowvar.est, rowvar, estnm, psenm, allin1=allin1,
-		char.width=char.width, estnull=estnull, psenull=psenull,
-		estround=estround, pseround=pseround)
+		             char.width=char.width, estnull=estnull, psenull=psenull,
+		             estround=estround, pseround=pseround)
 
       if (allin1) {
         estpse$Total <- estpse.row
@@ -516,9 +559,12 @@ crossxtab <- function (group.est, rowvar.est=NULL, colvar.est=NULL, total.est=NU
       }
     }
   } else if (!is.null(colvar.est) || !is.null(rowvar.est)) {
-    estpse.col <- crosstabx(colvar.est, colvar, estnm, psenm, allin1=allin1,
-		char.width=char.width, estnull=estnull, psenull=psenull,
-		estround=estround, pseround=pseround)
+
+    estpse.col <- crosstabx(colvar.est, colvar, estnm, psenm, 
+                            allin1 = allin1,
+		                        char.width = char.width, 
+		                        estnull = estnull, psenull = psenull,
+		                        estround = estround, pseround = pseround)
 
     if (allin1) {
       estpse <- rbind(setDF(estpse), c(totals, estpse.col))
@@ -526,10 +572,11 @@ crossxtab <- function (group.est, rowvar.est=NULL, colvar.est=NULL, total.est=NU
       est <- rbind(setDF(est), c(totals, estpse.col$est))
       pse <- rbind(setDF(pse), c(totals, estpse.col$pse))
     }
-
-    estpse.row <- crosstabx(rowvar.est, rowvar, estnm, psenm, allin1=allin1,
-		char.width=char.width, estnull=estnull, psenull=psenull,
-		estround=estround, pseround=pseround)
+    estpse.row <- crosstabx(rowvar.est, rowvar, estnm, psenm, 
+                            allin1 = allin1,
+		                        char.width = char.width, 
+		                        estnull = estnull, psenull = psenull,
+		                        estround = estround, pseround = pseround)
     if (!allin1) {
       est.row <- estpse.row$est
       pse.row <- estpse.row$pse
@@ -544,7 +591,7 @@ crossxtab <- function (group.est, rowvar.est=NULL, colvar.est=NULL, total.est=NU
           pse.tot <- psenull
         } else {
           estpse.tot <- paste0(format(psenull, big.mark=","), " (",
-			format(psenull, justify="right", width=char.width), ")")
+			         format(psenull, justify="right", width=char.width), ")")
         }
       } else {
         est.tot <- total.est[[estnm]]
@@ -552,14 +599,14 @@ crossxtab <- function (group.est, rowvar.est=NULL, colvar.est=NULL, total.est=NU
 
         if (allin1) {
           estpse.tot <- mapply(allin1f, est.tot, pse.tot,
-			MoreArgs=list(char.width=char.width, estnull=estnull, psenull=psenull,
-			estround=estround, pseround=pseround))
-		}
+			          MoreArgs=list(char.width=char.width, estnull=estnull, psenull=psenull,
+			                        estround=estround, pseround=pseround))
+        }
       }
     } else {
       if (allin1) {
         estpse.tot <- paste0(format(psenull, big.mark=","), " (", format(psenull,
-			justify="right", width=char.width), ")")
+			       justify="right", width=char.width), ")")
       } else {
         est.row <- as.character(est.row)
         est.tot <- 0
@@ -672,12 +719,12 @@ crossxbyunit <- function(unit=NULL, unit_grpest=NULL, unit_rowest=NULL,
   ## Get cross tables
   #########################################################
   tabs <- crossxtab(group.est=group.est, rowvar.est=rowvar.est,
-	colvar.est=colvar.est, total.est=total.est, rowvar=rowvar,
-	colvar=colvar, estnm=estnm, psenm=psenm, allin1=allin1,
-	rowgrp=rowgrp, rowgrpnm=rowgrpnm, title.rnames=title.rnames,
- 	estnull=estnull, psenull=psenull, char.width=char.width,
-	estround=estround, pseround=pseround)
-
+	             colvar.est=colvar.est, total.est=total.est, rowvar=rowvar,
+	             colvar=colvar, estnm=estnm, psenm=psenm, allin1=allin1,
+	             rowgrp=rowgrp, rowgrpnm=rowgrpnm, title.rnames=title.rnames,
+ 	             estnull=estnull, psenull=psenull, char.width=char.width,
+	             estround=estround, pseround=pseround)
+  
   if (allin1) {
     estpsetab <- tabs
     if (!is.null(estpsetab)) {
