@@ -1,8 +1,18 @@
 #' @rdname write2_desc
 #' @export
-write2sqlite <- function(layer, SQLitefn=NULL, out_name=NULL, gpkg=FALSE,
- 	outfolder=NULL, overwrite=FALSE, append_layer=FALSE, createnew=FALSE,
-	dbconn=NULL, dbconnopen=FALSE, index.unique=NULL, index=NULL){
+write2sqlite <- function(layer, 
+                         SQLitefn = NULL, 
+                         out_name = NULL, 
+                         lowernames = FALSE,
+                         gpkg = FALSE,
+                         outfolder = NULL, 
+                         overwrite = FALSE, 
+                         append_layer = FALSE, 
+                         createnew = FALSE,
+                         dbconn = NULL, 
+                         dbconnopen = FALSE, 
+                         index.unique = NULL, 
+                         index = NULL){
   ###################################################################################
   ## DESCRIPTION: Internal function to write to csv file.
   ##
@@ -40,7 +50,10 @@ write2sqlite <- function(layer, SQLitefn=NULL, out_name=NULL, gpkg=FALSE,
 
   ## Check layer
   layer <- pcheck.table(layer)
-
+  if (lowernames) {
+    names(layer) <- tolower(names(layer))
+  }
+  
 
   ## Change columns of type POSIX* to character before writing to database
   if (any(grepl("POSIX", lapply(layer, class)))) {
@@ -60,10 +73,18 @@ write2sqlite <- function(layer, SQLitefn=NULL, out_name=NULL, gpkg=FALSE,
     }
     for (i in 1:length(index.unique)) {
       indexu <- index.unique[[i]]
-	  
+      if (lowernames) {
+        indexu <- tolower(indexu)
+      }
+      
 	    if (!all(indexu %in% names(layer))) {
-        warning("invalid index.unique... names not in ", out_name)
-		    message(toString(indexu))
+	      indexuchk <- sapply(indexu, findnm, names(layer), returnNULL = TRUE)
+	      if (all(is.null(unlist(indexuchk)))) {
+          warning("invalid index.unique... names not in ", out_name)
+		      message(toString(indexu))
+	      } else {
+	        indexu <- indexuchk
+	      }
       }
 
       if (!all(indexu %in% names(layer))) {
@@ -96,8 +117,18 @@ write2sqlite <- function(layer, SQLitefn=NULL, out_name=NULL, gpkg=FALSE,
     }
     for (i in 1:length(index)) {
       indexi <- index[[i]]
+      if (lowernames) {
+        indexi <- tolower(indexi)
+      }
+      
       if (!all(indexi %in% names(layer))) {
-        warning("invalid index... names not in layer: ", toString(indexi))
+        indexichk <- sapply(indexi, findnm, names(layer), returnNULL = TRUE)
+        if (all(is.null(unlist(indexichk)))) {
+          warning("invalid index... names not in layer: ", toString(indexi))
+          message(toString(indexichk))
+        } else {
+          indexi <- indexichk
+        }
       } else {
 
         indxnm <- paste0(out_name, "_", paste(tolower(indexi), collapse="_"), "_idx")
